@@ -19,7 +19,138 @@
 
 #include "includes.h"
 #include "../lib/async_req/async_sock.h"
+#include "../lib/util/tevent_ntstatus.h"
+#include "../lib/util/tevent_unix.h"
+#include "client.h"
 #include "async_smb.h"
+#include "../libcli/smb/read_smb.h"
+#include "libsmb/nmblib.h"
+
+struct cli_session_request_state {
+	struct tevent_context *ev;
+	int sock;
+	uint32 len_hdr;
+	struct iovec iov[3];
+	uint8_t nb_session_response;
+};
+
+static void cli_session_request_sent(struct tevent_req *subreq);
+static void cli_session_request_recvd(struct tevent_req *subreq);
+
+static struct tevent_req *cli_session_request_send(TALLOC_CTX *mem_ctx,
+					struct tevent_context *ev,
+					int sock,
+					const struct nmb_name *called,
+					const struct nmb_name *calling)
+{
+	struct tevent_req *req, *subreq;
+	struct cli_session_request_state *state;
+
+	req = tevent_req_create(mem_ctx, &state,
+				struct cli_session_request_state);
+	if (req == NULL) {
+		return NULL;
+	}
+	state->ev = ev;
+	state->sock = sock;
+
+	state->iov[1].iov_base = name_mangle(
+		state, called->name, called->name_type);
+	if (tevent_req_nomem(state->iov[1].iov_base, req)) {
+		return tevent_req_post(req, ev);
+	}
+	state->iov[1].iov_len = name_len(
+		(unsigned char *)state->iov[1].iov_base,
+		talloc_get_size(state->iov[1].iov_base));
+
+	state->iov[2].iov_base = name_mangle(
+		state, calling->name, calling->name_type);
+	if (tevent_req_nomem(state->iov[2].iov_base, req)) {
+		return tevent_req_post(req, ev);
+	}
+	state->iov[2].iov_len = name_len(
+		(unsigned char *)state->iov[2].iov_base,
+		talloc_get_size(state->iov[2].iov_base));
+
+	_smb_setlen(((char *)&state->len_hdr),
+		    state->iov[1].iov_len + state->iov[2].iov_len);
+	SCVAL((char *)&state->len_hdr, 0, 0x81);
+
+	state->iov[0].iov_base = &state->len_hdr;
+	state->iov[0].iov_len = sizeof(state->len_hdr);
+
+	subreq = writev_send(state, ev, NULL, sock, true, state->iov, 3);
+	if (tevent_req_nomem(subreq, req)) {
+		return tevent_req_post(req, ev);
+	}
+	tevent_req_set_callback(subreq, cli_session_request_sent, req);
+	return req;
+}
+
+static void cli_session_request_sent(struct tevent_req *subreq)
+{
+	struct tevent_req *req = tevent_req_callback_data(
+		subreq, struct tevent_req);
+	struct cli_session_request_state *state = tevent_req_data(
+		req, struct cli_session_request_state);
+	ssize_t ret;
+	int err;
+
+	ret = writev_recv(subreq, &err);
+	TALLOC_FREE(subreq);
+	if (ret == -1) {
+		tevent_req_error(req, err);
+		return;
+	}
+	subreq = read_smb_send(state, state->ev, state->sock);
+	if (tevent_req_nomem(subreq, req)) {
+		return;
+	}
+	tevent_req_set_callback(subreq, cli_session_request_recvd, req);
+}
+
+static void cli_session_request_recvd(struct tevent_req *subreq)
+{
+	struct tevent_req *req = tevent_req_callback_data(
+		subreq, struct tevent_req);
+	struct cli_session_request_state *state = tevent_req_data(
+		req, struct cli_session_request_state);
+	uint8_t *buf;
+	ssize_t ret;
+	int err;
+
+	ret = read_smb_recv(subreq, talloc_tos(), &buf, &err);
+	TALLOC_FREE(subreq);
+
+	if (ret < 4) {
+		ret = -1;
+		err = EIO;
+	}
+	if (ret == -1) {
+		tevent_req_error(req, err);
+		return;
+	}
+	/*
+	 * In case of an error there is more information in the data
+	 * portion according to RFC1002. We're not subtle enough to
+	 * respond to the different error conditions, so drop the
+	 * error info here.
+	 */
+	state->nb_session_response = CVAL(buf, 0);
+	tevent_req_done(req);
+}
+
+static bool cli_session_request_recv(struct tevent_req *req, int *err, uint8_t *resp)
+{
+	struct cli_session_request_state *state = tevent_req_data(
+		req, struct cli_session_request_state);
+
+	if (tevent_req_is_unix_error(req, err)) {
+		return false;
+	}
+	*resp = state->nb_session_response;
+	return true;
+}
 
 struct nb_connect_state {
 	struct tevent_context *ev;
@@ -60,7 +191,7 @@ static struct tevent_req *nb_connect_send(TALLOC_CTX *mem_ctx,
 
 	talloc_set_destructor(state, nb_connect_state_destructor);
 
-	subreq = open_socket_out_send(state, ev, addr, 139, 5000);
+	subreq = open_socket_out_send(state, ev, addr, NBT_SMB_PORT, 5000);
 	if (tevent_req_nomem(subreq, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -145,7 +276,7 @@ static void nb_connect_done(struct tevent_req *subreq)
 		make_nmb_name(&state->called, state->called_name, 0x20);
 
 		subreq = open_socket_out_send(state, state->ev, state->addr,
-					      139, 5000);
+					      NBT_SMB_PORT, 5000);
 		if (tevent_req_nomem(subreq, req)) {
 			return;
 		}
@@ -176,7 +307,9 @@ struct smbsock_connect_state {
 	struct tevent_context *ev;
 	const struct sockaddr_storage *addr;
 	const char *called_name;
+	uint8_t called_type;
 	const char *calling_name;
+	uint8_t calling_type;
 	struct tevent_req *req_139;
 	struct tevent_req *req_445;
 	int sock;
@@ -191,10 +324,13 @@ static void smbsock_connect_do_139(struct tevent_req *subreq);
 struct tevent_req *smbsock_connect_send(TALLOC_CTX *mem_ctx,
 					struct tevent_context *ev,
 					const struct sockaddr_storage *addr,
+					uint16_t port,
 					const char *called_name,
-					const char *calling_name)
+					int called_type,
+					const char *calling_name,
+					int calling_type)
 {
-	struct tevent_req *req, *subreq;
+	struct tevent_req *req;
 	struct smbsock_connect_state *state;
 
 	req = tevent_req_create(mem_ctx, &state, struct smbsock_connect_state);
@@ -206,12 +342,44 @@ struct tevent_req *smbsock_connect_send(TALLOC_CTX *mem_ctx,
 	state->sock = -1;
 	state->called_name =
 		(called_name != NULL) ? called_name : "*SMBSERVER";
+	state->called_type =
+		(called_type != -1) ? called_type : 0x20;
 	state->calling_name =
-		(calling_name != NULL) ? calling_name : global_myname();
+		(calling_name != NULL) ? calling_name : lp_netbios_name();
+	state->calling_type =
+		(calling_type != -1) ? calling_type : 0x00;
 
 	talloc_set_destructor(state, smbsock_connect_state_destructor);
 
-	state->req_445 = open_socket_out_send(state, ev, addr, 445, 5000);
+	if (port == NBT_SMB_PORT) {
+		state->req_139 = nb_connect_send(state, state->ev, state->addr,
+						 state->called_name,
+						 state->called_type,
+						 state->calling_name,
+						 state->calling_type);
+		if (tevent_req_nomem(state->req_139, req)) {
+			return tevent_req_post(req, ev);
+		}
+		tevent_req_set_callback(
+			state->req_139, smbsock_connect_connected, req);
+		return req;
+	}
+	if (port != 0) {
+		state->req_445 = open_socket_out_send(state, ev, addr, port,
+						      5000);
+		if (tevent_req_nomem(state->req_445, req)) {
+			return tevent_req_post(req, ev);
+		}
+		tevent_req_set_callback(
+			state->req_445, smbsock_connect_connected, req);
+		return req;
+	}
+
+	/*
+	 * port==0, try both
+	 */
+
+	state->req_445 = open_socket_out_send(state, ev, addr, TCP_SMB_PORT, 5000);
 	if (tevent_req_nomem(state->req_445, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -219,7 +387,7 @@ struct tevent_req *smbsock_connect_send(TALLOC_CTX *mem_ctx,
 				req);
 
 	/*
-	 * After 5 msecs, fire the 139 request
+	 * After 5 msecs, fire the 139 (NBT) request
 	 */
 	state->req_139 = tevent_wakeup_send(
 		state, ev, timeval_current_ofs(0, 5000));
@@ -237,6 +405,7 @@ static int smbsock_connect_state_destructor(
 {
 	if (state->sock != -1) {
 		close(state->sock);
+		state->sock = -1;
 	}
 	return 0;
 }
@@ -256,8 +425,10 @@ static void smbsock_connect_do_139(struct tevent_req *subreq)
 		return;
 	}
 	state->req_139 = nb_connect_send(state, state->ev, state->addr,
-					 state->called_name, 0x20,
-					 state->calling_name, 0x0);
+					 state->called_name,
+					 state->called_type,
+					 state->calling_name,
+					 state->calling_type);
 	if (tevent_req_nomem(state->req_139, req)) {
 		return;
 	}
@@ -279,14 +450,14 @@ static void smbsock_connect_connected(struct tevent_req *subreq)
 		status = open_socket_out_recv(subreq, &state->sock);
 		TALLOC_FREE(state->req_445);
 		unfinished_req = state->req_139;
-		state->port = 445;
+		state->port = TCP_SMB_PORT;
 
 	} else if (subreq == state->req_139) {
 
 		status = nb_connect_recv(subreq, &state->sock);
 		TALLOC_FREE(state->req_139);
 		unfinished_req = state->req_445;
-		state->port = 139;
+		state->port = NBT_SMB_PORT;
 
 	} else {
 		tevent_req_nterror(req, NT_STATUS_INTERNAL_ERROR);
@@ -313,7 +484,7 @@ static void smbsock_connect_connected(struct tevent_req *subreq)
 }
 
 NTSTATUS smbsock_connect_recv(struct tevent_req *req, int *sock,
-			  uint16_t *port)
+			      uint16_t *ret_port)
 {
 	struct smbsock_connect_state *state = tevent_req_data(
 		req, struct smbsock_connect_state);
@@ -324,33 +495,41 @@ NTSTATUS smbsock_connect_recv(struct tevent_req *req, int *sock,
 	}
 	*sock = state->sock;
 	state->sock = -1;
-	if (port != NULL) {
-		*port = state->port;
+	if (ret_port != NULL) {
+		*ret_port = state->port;
 	}
 	return NT_STATUS_OK;
 }
 
-NTSTATUS smbsock_connect(const struct sockaddr_storage *addr,
-			 const char *called_name, const char *calling_name,
-			 int *pfd, uint16_t *port)
+NTSTATUS smbsock_connect(const struct sockaddr_storage *addr, uint16_t port,
+			 const char *called_name, int called_type,
+			 const char *calling_name, int calling_type,
+			 int *pfd, uint16_t *ret_port, int sec_timeout)
 {
 	TALLOC_CTX *frame = talloc_stackframe();
-	struct event_context *ev;
+	struct tevent_context *ev;
 	struct tevent_req *req;
 	NTSTATUS status = NT_STATUS_NO_MEMORY;
 
-	ev = event_context_init(frame);
+	ev = tevent_context_init(frame);
 	if (ev == NULL) {
 		goto fail;
 	}
-	req = smbsock_connect_send(frame, ev, addr, called_name, calling_name);
+	req = smbsock_connect_send(frame, ev, addr, port,
+				   called_name, called_type,
+				   calling_name, calling_type);
 	if (req == NULL) {
 		goto fail;
 	}
+	if ((sec_timeout != 0) &&
+	    !tevent_req_set_endtime(
+		    req, ev, timeval_current_ofs(sec_timeout, 0))) {
+		goto fail;
+	}
 	if (!tevent_req_poll_ntstatus(req, ev, &status)) {
 		goto fail;
 	}
-	status = smbsock_connect_recv(req, pfd, port);
+	status = smbsock_connect_recv(req, pfd, ret_port);
  fail:
 	TALLOC_FREE(frame);
 	return status;
@@ -360,14 +539,18 @@ struct smbsock_any_connect_state {
 	struct tevent_context *ev;
 	const struct sockaddr_storage *addrs;
 	const char **called_names;
+	int *called_types;
+	const char **calling_names;
+	int *calling_types;
 	size_t num_addrs;
+	uint16_t port;
 
 	struct tevent_req **requests;
 	size_t num_sent;
 	size_t num_received;
 
 	int fd;
-	uint16_t port;
+	uint16_t chosen_port;
 	size_t chosen_index;
 };
 
@@ -380,7 +563,10 @@ struct tevent_req *smbsock_any_connect_send(TALLOC_CTX *mem_ctx,
 					    struct tevent_context *ev,
 					    const struct sockaddr_storage *addrs,
 					    const char **called_names,
-					    size_t num_addrs)
+					    int *called_types,
+					    const char **calling_names,
+					    int *calling_types,
+					    size_t num_addrs, uint16_t port)
 {
 	struct tevent_req *req, *subreq;
 	struct smbsock_any_connect_state *state;
@@ -394,6 +580,10 @@ struct tevent_req *smbsock_any_connect_send(TALLOC_CTX *mem_ctx,
 	state->addrs = addrs;
 	state->num_addrs = num_addrs;
 	state->called_names = called_names;
+	state->called_types = called_types;
+	state->calling_names = calling_names;
+	state->calling_types = calling_types;
+	state->port = port;
 
 	if (num_addrs == 0) {
 		tevent_req_nterror(req, NT_STATUS_INVALID_PARAMETER);
@@ -458,9 +648,15 @@ static bool smbsock_any_connect_send_next(
 	}
 	subreq = smbsock_connect_send(
 		state->requests, state->ev, &state->addrs[state->num_sent],
+		state->port,
 		(state->called_names != NULL)
 		? state->called_names[state->num_sent] : NULL,
-		NULL);
+		(state->called_types != NULL)
+		? state->called_types[state->num_sent] : -1,
+		(state->calling_names != NULL)
+		? state->calling_names[state->num_sent] : NULL,
+		(state->calling_types != NULL)
+		? state->calling_types[state->num_sent] : -1);
 	if (tevent_req_nomem(subreq, req)) {
 		return false;
 	}
@@ -480,7 +676,7 @@ static void smbsock_any_connect_connected(struct tevent_req *subreq)
 		req, struct smbsock_any_connect_state);
 	NTSTATUS status;
 	int fd;
-	uint16_t port;
+	uint16_t chosen_port;
 	size_t i;
 	size_t chosen_index = 0;
 
@@ -495,7 +691,7 @@ static void smbsock_any_connect_connected(struct tevent_req *subreq)
 		return;
 	}
 
-	status = smbsock_connect_recv(subreq, &fd, &port);
+	status = smbsock_connect_recv(subreq, &fd, &chosen_port);
 
 	TALLOC_FREE(subreq);
 	state->requests[chosen_index] = NULL;
@@ -506,14 +702,14 @@ static void smbsock_any_connect_connected(struct tevent_req *subreq)
 		 */
 		TALLOC_FREE(state->requests);
 		state->fd = fd;
-		state->port = port;
+		state->chosen_port = chosen_port;
 		state->chosen_index = chosen_index;
 		tevent_req_done(req);
 		return;
 	}
 
 	state->num_received += 1;
-	if (state->num_received <= state->num_addrs) {
+	if (state->num_received < state->num_addrs) {
 		/*
 		 * More addrs pending, wait for the others
 		 */
@@ -528,7 +724,8 @@ static void smbsock_any_connect_connected(struct tevent_req *subreq)
 }
 
 NTSTATUS smbsock_any_connect_recv(struct tevent_req *req, int *pfd,
-				  size_t *chosen_index, uint16_t *port)
+				  size_t *chosen_index,
+				  uint16_t *chosen_port)
 {
 	struct smbsock_any_connect_state *state = tevent_req_data(
 		req, struct smbsock_any_connect_state);
@@ -541,34 +738,48 @@ NTSTATUS smbsock_any_connect_recv(struct tevent_req *req, int *pfd,
 	if (chosen_index != NULL) {
 		*chosen_index = state->chosen_index;
 	}
-	if (port != NULL) {
-		*port = state->port;
+	if (chosen_port != NULL) {
+		*chosen_port = state->chosen_port;
 	}
 	return NT_STATUS_OK;
 }
 
 NTSTATUS smbsock_any_connect(const struct sockaddr_storage *addrs,
-			     const char **called_names, size_t num_addrs,
-			     int *pfd, size_t *chosen_index, uint16_t *port)
+			     const char **called_names,
+			     int *called_types,
+			     const char **calling_names,
+			     int *calling_types,
+			     size_t num_addrs,
+			     uint16_t port,
+			     int sec_timeout,
+			     int *pfd, size_t *chosen_index,
+			     uint16_t *chosen_port)
 {
 	TALLOC_CTX *frame = talloc_stackframe();
-	struct event_context *ev;
+	struct tevent_context *ev;
 	struct tevent_req *req;
 	NTSTATUS status = NT_STATUS_NO_MEMORY;
 
-	ev = event_context_init(frame);
+	ev = tevent_context_init(frame);
 	if (ev == NULL) {
 		goto fail;
 	}
-	req = smbsock_any_connect_send(frame, ev, addrs, called_names,
-				       num_addrs);
+	req = smbsock_any_connect_send(frame, ev, addrs,
+				       called_names, called_types,
+				       calling_names, calling_types,
+				       num_addrs, port);
 	if (req == NULL) {
 		goto fail;
 	}
+	if ((sec_timeout != 0) &&
+	    !tevent_req_set_endtime(
+		    req, ev, timeval_current_ofs(sec_timeout, 0))) {
+		goto fail;
+	}
 	if (!tevent_req_poll_ntstatus(req, ev, &status)) {
 		goto fail;
 	}
-	status = smbsock_any_connect_recv(req, pfd, chosen_index, port);
+	status = smbsock_any_connect_recv(req, pfd, chosen_index, chosen_port);
  fail:
 	TALLOC_FREE(frame);
 	return status;
